@@ -1,9 +1,10 @@
-use std::{str::FromStr, sync::mpsc::Sender};
+use std::str::FromStr;
 
 use clap::{value_t, ArgMatches};
 use oura::{
     framework::*,
-    sources::common::{AddressArg, BearerKind, MagicArg},
+    mapping::MapperConfig,
+    sources::common::{AddressArg, BearerKind, MagicArg, PointArg},
 };
 
 use serde_derive::Deserialize;
@@ -35,10 +36,10 @@ enum WatchSource {
 }
 
 impl SourceConfig for WatchSource {
-    fn bootstrap(&self, output: Sender<Event>) -> BootstrapResult {
+    fn bootstrap(&self) -> PartialBootstrapResult {
         match self {
-            WatchSource::N2C(c) => c.bootstrap(output),
-            WatchSource::N2N(c) => c.bootstrap(output),
+            WatchSource::N2C(c) => c.bootstrap(),
+            WatchSource::N2N(c) => c.bootstrap(),
         }
     }
 }
@@ -56,6 +57,11 @@ pub fn run(args: &ArgMatches) -> Result<(), Error> {
         false => None,
     };
 
+    let since = match args.is_present("since") {
+        true => Some(value_t!(args, "since", PointArg)?),
+        false => None,
+    };
+
     let mode = match (args.is_present("mode"), &bearer) {
         (true, _) => value_t!(args, "mode", PeerMode).expect("invalid value for 'mode' arg"),
         (false, BearerKind::Tcp) => PeerMode::AsNode,
@@ -67,23 +73,25 @@ pub fn run(args: &ArgMatches) -> Result<(), Error> {
             address: AddressArg(bearer, socket),
             magic,
             well_known: None,
+            mapper: MapperConfig::default(),
+            since,
         }),
         PeerMode::AsClient => WatchSource::N2C(N2CConfig {
             address: AddressArg(bearer, socket),
             magic,
             well_known: None,
+            mapper: MapperConfig::default(),
+            since,
         }),
     };
 
     let sink_setup = oura::sinks::terminal::Config::default();
 
-    let (tx, rx) = std::sync::mpsc::channel();
+    let (source_handle, source_output) = source_setup.bootstrap()?;
+    let sink_handle = sink_setup.bootstrap(source_output)?;
 
-    let source = source_setup.bootstrap(tx)?;
-    let sink = sink_setup.bootstrap(rx)?;
-
-    sink.join().map_err(|_| "error in sink thread")?;
-    source.join().map_err(|_| "error in source thread")?;
+    sink_handle.join().map_err(|_| "error in sink thread")?;
+    source_handle.join().map_err(|_| "error in source thread")?;
 
     Ok(())
 }
