@@ -2,7 +2,10 @@ use std::fmt::{Display, Write};
 
 use crossterm::style::{Attribute, Color, Stylize};
 
-use crate::framework::{Event, EventData};
+use crate::framework::{
+    Event, EventData, MetadataRecord, MintRecord, OutputAssetRecord, TransactionRecord,
+    TxInputRecord, TxOutputRecord,
+};
 
 pub struct LogLine {
     prefix: &'static str,
@@ -18,47 +21,61 @@ impl LogLine {
             EventData::Block {
                 body_size,
                 issuer_vkey,
-            } => LogLine {
-                prefix: "BLOCK",
-                color: Color::Magenta,
-                content: format!(
-                    "{{ body size: {}, issues vkey: {}, timestamp: {} }}",
+                tx_count,
+                ..
+            } => {
+                LogLine {
+                    prefix: "BLOCK",
+                    color: Color::Magenta,
+                    content: format!(
+                    "{{ hash: {}, body size: {}, tx_count: {}, issuer vkey: {}, timestamp: {} }}",
+                    &source.context.block_hash.as_ref().unwrap_or(&"".to_string()),
                     body_size,
+                    tx_count,
                     issuer_vkey,
                     source.context.timestamp.unwrap_or_default(),
                 ),
-                source,
-                max_width,
-            },
-            EventData::Transaction { fee, ttl, .. } => LogLine {
+                    source,
+                    max_width,
+                }
+            }
+            EventData::Transaction(TransactionRecord {
+                total_output,
+                fee,
+                ttl,
+                ..
+            }) => LogLine {
                 prefix: "TX",
                 color: Color::DarkBlue,
                 content: format!(
-                    "{{ fee: {}, hash: {:?}, ttl: {:?} }}",
-                    fee, &source.context.tx_hash, ttl
+                    "{{ total_output: {}, fee: {}, hash: {:?}, ttl: {:?} }}",
+                    total_output, fee, &source.context.tx_hash, ttl
                 ),
                 source,
                 max_width,
             },
-            EventData::TxInput { tx_id, index } => LogLine {
+            EventData::TxInput(TxInputRecord { tx_id, index }) => LogLine {
                 prefix: "STXI",
                 color: Color::Blue,
                 content: format!("{{ tx_id: {}, index: {} }}", tx_id, index),
                 source,
                 max_width,
             },
-            EventData::TxOutput { address, amount } => LogLine {
+            EventData::TxOutput(TxOutputRecord {
+                address, amount, ..
+            }) => LogLine {
                 prefix: "UTXO",
                 color: Color::Blue,
                 content: format!("{{ to: {}, amount: {} }}", address, amount),
                 source,
                 max_width,
             },
-            EventData::OutputAsset {
+            EventData::OutputAsset(OutputAssetRecord {
                 policy,
                 asset,
                 amount,
-            } => LogLine {
+                ..
+            }) => LogLine {
                 prefix: "ASSET",
                 color: Color::Green,
                 content: format!(
@@ -68,7 +85,7 @@ impl LogLine {
                 source,
                 max_width,
             },
-            EventData::Metadata { key, subkey, value } => LogLine {
+            EventData::Metadata(MetadataRecord { key, subkey, value }) => LogLine {
                 prefix: "META",
                 color: Color::Yellow,
                 content: format!(
@@ -78,11 +95,11 @@ impl LogLine {
                 source,
                 max_width,
             },
-            EventData::Mint {
+            EventData::Mint(MintRecord {
                 policy,
                 asset,
                 quantity,
-            } => LogLine {
+            }) => LogLine {
                 prefix: "MINT",
                 color: Color::DarkGreen,
                 content: format!(
@@ -92,21 +109,14 @@ impl LogLine {
                 source,
                 max_width,
             },
-            EventData::NewNativeScript => LogLine {
-                prefix: "NATIVE+",
+            EventData::NativeScript => LogLine {
+                prefix: "NATIVE",
                 color: Color::White,
                 content: "{{ ... }}".to_string(),
                 source,
                 max_width,
             },
-            EventData::NewPlutusScript { data } => LogLine {
-                prefix: "PLUTUS+",
-                color: Color::White,
-                content: format!("{{ {} }}", data),
-                source,
-                max_width,
-            },
-            EventData::PlutusScriptRef { data } => LogLine {
+            EventData::PlutusScript { data } => LogLine {
                 prefix: "PLUTUS",
                 color: Color::White,
                 content: format!("{{ {} }}", data),
