@@ -6,7 +6,7 @@ use crate::framework::{BootstrapResult, Event, SinkConfig};
 
 use super::run::reducer_loop;
 
-const THROTTLE_MIN_SPAN_MILLIS: u64 = 500;
+const THROTTLE_MIN_SPAN_MILLIS: u64 = 300;
 
 #[derive(Default, Debug, Deserialize)]
 pub struct Config {
@@ -20,7 +20,9 @@ impl SinkConfig for Config {
                 .unwrap_or(THROTTLE_MIN_SPAN_MILLIS),
         );
 
-        let handle = std::thread::spawn(move || reducer_loop(throttle_min_span, input).unwrap());
+        let handle = std::thread::spawn(move || {
+            reducer_loop(throttle_min_span, input).expect("terminal sink loop failed");
+        });
 
         Ok(handle)
     }
