@@ -1,4 +1,4 @@
-use std::{net::TcpStream, ops::Deref, os::unix::net::UnixStream, sync::mpsc::Sender};
+use std::{net::TcpStream, ops::Deref, os::unix::net::UnixStream, sync::mpsc};
 
 use net2::TcpStreamExt;
 
@@ -6,16 +6,17 @@ use log::info;
 
 use pallas::ouroboros::network::{
     handshake::{n2n, MAINNET_MAGIC},
-    machines::run_agent,
+    machines::{primitives::Point, run_agent},
     multiplexer::{Channel, Multiplexer},
 };
 
 use serde_derive::Deserialize;
 
 use crate::{
-    framework::{BootstrapResult, ChainWellKnownInfo, Error, Event, SourceConfig},
+    framework::{ChainWellKnownInfo, Error, EventWriter, PartialBootstrapResult, SourceConfig},
+    mapping::MapperConfig,
     sources::{
-        common::{find_end_of_chain, AddressArg, BearerKind, MagicArg},
+        common::{find_end_of_chain, AddressArg, BearerKind, MagicArg, PointArg},
         n2n::{fetch_blocks_forever, observe_headers_forever},
     },
 };
@@ -27,7 +28,12 @@ pub struct Config {
     #[serde(deserialize_with = "crate::sources::common::deserialize_magic_arg")]
     pub magic: Option<MagicArg>,
 
+    pub since: Option<PointArg>,
+
     pub well_known: Option<ChainWellKnownInfo>,
+
+    #[serde(default)]
+    pub mapper: MapperConfig,
 }
 
 fn do_handshake(channel: &mut Channel, magic: u64) -> Result<(), Error> {
@@ -56,7 +62,9 @@ fn setup_tcp_multiplexer(address: &str) -> Result<Multiplexer, Error> {
 }
 
 impl SourceConfig for Config {
-    fn bootstrap(&self, output: Sender<Event>) -> BootstrapResult {
+    fn bootstrap(&self) -> PartialBootstrapResult {
+        let (output_tx, output_rx) = mpsc::channel();
+
         let mut muxer = match self.address.0 {
             BearerKind::Tcp => setup_tcp_multiplexer(&self.address.1)?,
             BearerKind::Unix => setup_unix_multiplexer(&self.address.1)?,
@@ -72,31 +80,35 @@ impl SourceConfig for Config {
             None => ChainWellKnownInfo::try_from_magic(magic)?,
         };
 
+        let writer = EventWriter::new(output_tx, well_known.clone().into(), self.mapper.clone());
+
         let mut hs_channel = muxer.use_channel(0);
         do_handshake(&mut hs_channel, magic)?;
 
         let mut cs_channel = muxer.use_channel(2);
 
-        let node_tip = find_end_of_chain(&mut cs_channel, &well_known)?;
+        let since: Point = match &self.since {
+            Some(arg) => arg.try_into()?,
+            None => find_end_of_chain(&mut cs_channel, &well_known)?,
+        };
 
-        info!("node tip: {:?}", &node_tip);
+        info!("starting from chain point: {:?}", &since);
 
         let (headers_tx, headers_rx) = std::sync::mpsc::channel();
 
-        let cs_events = output.clone();
-        let cs_chain_info = well_known.clone();
+        let cs_writer = writer.clone();
         let cs_handle = std::thread::spawn(move || {
-            observe_headers_forever(cs_channel, cs_chain_info, node_tip, cs_events, headers_tx)
+            observe_headers_forever(cs_channel, cs_writer, since, headers_tx)
                 .expect("chainsync loop failed");
         });
 
         let bf_channel = muxer.use_channel(3);
-        let bf_chain_info = well_known.clone();
+        let bf_writer = writer;
         let _bf_handle = std::thread::spawn(move || {
-            fetch_blocks_forever(bf_channel, bf_chain_info, headers_rx, output)
+            fetch_blocks_forever(bf_channel, bf_writer, headers_rx)
                 .expect("blockfetch loop failed");
         });
 
-        Ok(cs_handle)
+        Ok((cs_handle, output_rx))
     }
 }
